from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from djangovideos.models import Video

class VideoForm(forms.ModelForm):
    url = forms.CharField(label=_('Video url'))

    def clean_url(self):
        data = self.cleaned_data['url'].strip()
        if 'src=' in data:
            data = data.split('src=', 1)[1]
            data = data.split(' ')[0]
            data = data.strip(''''"''')
        if data.startswith('http://www.youtube.com/watch?v='):
            data = data.replace('http://www.youtube.com/watch?v=', 'http://www.youtube.com/v/')
        if data.startswith('http'):
            return data
        else:
            raise forms.ValidationError("No valid link found")

    class Meta:
        model = Video
        fields = ('url',)

    def save(self, request, obj, *args, **kwargs):
        self.instance.creator = request.user
        self.instance.content_type = ContentType.objects.get_for_model(obj)
        self.instance.object_id = obj.id
        super(VideoForm, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
import os

try:
    from pluggableapp import PluggableApp
except ImportError:
    # pluggableapp is not installed
    pass


def pluggableapp(**kw):
    app = PluggableApp('djangovideos', distribution='django-videos', **kw)
    app.append_app()
    app.append("TEMPLATE_CONTEXT_PROCESSORS", "django.core.context_processors.i18n",
                                              "django.core.context_processors.request")
    app.register_pattern('', r'^videos/', 'djangovideos.urls')
    app.insert_templates(__file__)
    return app




from django.template import Library, Node, Variable
from django.template.loader import get_template
from django.template import Context
from django.core.urlresolvers import reverse
from djangovideos.views import add_url_for_obj
from djangovideos.forms import VideoForm
from djangovideos.models import Video

register = Library()

@register.inclusion_tag('djangovideos/add_form.html', takes_context=True)
def video_form(context, obj):
    """
    Renders a "upload video" form.

    The user must own ``videos.add_video permission`` to add
    videos.
    """
    if context['user'].has_perm('djangovideos.add_video'):
        return {
            'form': VideoForm(),
            'form_url': add_url_for_obj(obj),
            'next': context['request'].build_absolute_uri(),
        }
    else:
        return {
            'form': None,
        }

@register.inclusion_tag('djangovideos/delete_link.html', takes_context=True)
def video_delete_link(context, video):
    """
    Renders a html link to the delete view of the given video. Returns
    no content if the request-user has no permission to delete videos.

    The user must own either the ``videos.delete_video`` permission
    and is the creator of the video, that he can delete it or he has
    ``videos.delete_foreign_videos`` which allows him to delete all
    videos.
    """
    if context['user'].has_perm('delete_foreign_videos') \
       or (context['user'] == video.creator and \
           context['user'].has_perm('videos.delete_video')):
        return {
            'next': context['request'].build_absolute_uri(),
            'delete_url': reverse('delete_video', kwargs={'video_pk': video.pk})
        }
    return {'delete_url': None,}



class VideosForObjectNode(Node):
    def __init__(self, obj, var_name):
        self.obj = obj
        self.var_name = var_name

    def resolve(self, var, context):
        """Resolves a variable out of context if it's not in quotes"""
        if var[0] in ('"', "'") and var[-1] == var[0]:
            return var[1:-1]
        else:
            return Variable(var).resolve(context)

    def render(self, context):
        obj = self.resolve(self.obj, context)
        var_name = self.resolve(self.var_name, context)
        context[var_name] = Video.objects.videos_for_object(obj)
        return ''

@register.tag
def get_videos_for(parser, token):
    """
    Resolves videos that are attached to a given object. You can specify
    the variable name in the context the videos are stored using the `as`
    argument. Default context variable name is `videos`.

    Syntax::

        {% get_videos_for obj %}
        {% for att in videos %}
            {{ att }}
        {% endfor %}

        {% get_videos_for obj as "my_videos" %}

    """
    def next_bit_for(bits, key, if_none=None):
        try:
            return bits[bits.index(key)+1]
        except ValueError:
            return if_none

    bits = token.contents.split()
    args = {
        'obj': next_bit_for(bits, 'get_videos_for'),
        'var_name': next_bit_for(bits, 'as', '"videos"'),
    }
    return VideosForObjectNode(**args)

class VideoNode(Node):
    def __init__(self, obj, size):
        self.obj = obj
        self.size = size

    def resolve(self, var, context):
        """Resolves a variable out of context if it's not in quotes"""
        if var[0] in ('"', "'") and var[-1] == var[0]:
            return var[1:-1]
        else:
            return Variable(var).resolve(context)

    def render(self, context):
        obj = self.resolve(self.obj, context)
        width, height = self.size.split('x')
        t = get_template('djangovideos/video.html')
        return t.render(Context(dict(object=obj, width=width, height=height)))

@register.tag
def embed_video(parser, token):
    """
    Resolves videos that are attached to a given object. You can specify
    the variable name in the context the videos are stored using the `as`
    argument. Default context variable name is `videos`.

    Syntax::

        {% video obj %}

    """

    size = '640x505'
    bits = token.contents.split()
    if len(bits) == 3:
        name, obj, size = bits
    else:
        name, obj = bits
    args = {
        'obj': obj,
        'size': size,
    }
    return VideoNode(**args)



from django.shortcuts import render_to_response, get_object_or_404
from django.views.decorators.http import require_POST
from django.http import HttpResponseRedirect
from django.db.models.loading import get_model
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext, ugettext_lazy as _
from django.template.context import RequestContext
from django.contrib.auth.decorators import login_required
from djangovideos.models import Video
from djangovideos.forms import VideoForm

def add_url_for_obj(obj):
    return reverse('add_video', kwargs={
                        'app_label': obj._meta.app_label,
                        'module_name': obj._meta.module_name,
                        'pk': obj.pk
                    })

@require_POST
@login_required
def add_video(request, app_label, module_name, pk,
                   template_name='djangovideos/add.html', extra_context={}):

    next = request.POST.get('next', '/')
    model = get_model(app_label, module_name)
    if model is None:
        return HttpResponseRedirect(next)
    obj = get_object_or_404(model, pk=pk)
    form = VideoForm(request.POST)

    if form.is_valid():
        form.save(request, obj)
        request.user.message_set.create(message=ugettext('Your video was uploaded.'))
        return HttpResponseRedirect(next)
    else:
        template_context = {
            'form': form,
            'form_url': add_url_for_obj(obj),
            'next': next,
        }
        template_context.update(extra_context)
        return render_to_response(template_name, template_context,
                                  RequestContext(request))

@login_required
def delete_video(request, video_pk):
    g = get_object_or_404(video, pk=video_pk)
    if request.user.has_perm('delete_foreign_videos') \
       or request.user == g.creator:
        g.delete()
        request.user.message_set.create(message=ugettext('Your video was deleted.'))
    next = request.REQUEST.get('next') or '/'
    return HttpResponseRedirect(next)



# -*- coding: utf-8 -*-
from djangovideos.tests.functionnal import *
from djangovideos.models import Video

class TestFlashUrl(UserWebTest):

    value = "http://www.youtube.com/watch?v=ZFeHUdEQ1Zs"

    def test_index(self):
        resp = self.app.get(reverse('index'), user='user1')
        resp.mustcontain('submit')
        form = resp.form
        form['url'] = self.value
        resp = form.submit()
        resp = resp.follow()
        resp.mustcontain('<param name="movie" value="http://www.youtube.com/v/ZFeHUdEQ1Zs')

class TestObject(TestFlashUrl):

    value = '<object width="640" height="505"><param name="movie" value="http://www.youtube.com/v/ZFeHUdEQ1Zs&hl=fr_FR&fs=1&"></param><param name="allowFullScreen" value="true"></param><param name="allowscriptaccess" value="always"></param><embed src="http://www.youtube.com/v/ZFeHUdEQ1Zs&hl=fr_FR&fs=1&" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" width="640" height="505"></embed></object>'



from datetime import datetime
import os
from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _

class VideoManager(models.Manager):
    def videos_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.id)

class Video(models.Model):

    objects = VideoManager()

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    creator = models.ForeignKey(User, related_name="created_videos", verbose_name=_('creator'))
    url = models.CharField(_('url'), max_length=255)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)


    class Meta:
        ordering = ['-created']
        permissions = (
            ('delete_foreign_videos', 'Can delete foreign videos'),
        )

    def __unicode__(self):
        return '%s - %s' % (self.creator.username, self.url)




from setuptools import setup, find_packages
import sys, os

def read(*names):
    values = dict()
    for name in names:
        filename = name+'.txt'
        if os.path.isfile(filename):
            value = open(name+'.txt').read()
        else:
            value = ''
        values[name] = value
    return values

long_description="""
%(README)s

See http://packages.python.org/django-videos/ for the full documentation

News
====

%(CHANGES)s

""" % read('README', 'CHANGES')

version = '0.1'

setup(name='django-videos',
      version=version,
      description="Add some youtube videos to your django app",
      long_description=long_description,
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='django youtube',
      author='Bearstech',
      author_email='gpasgrimaud@bearstech.com',
      url='http://packages.python.org/django-videos/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'test_project']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      [django.pluggable_app]
      djangovideos = djangovideos:pluggableapp
      """,
      )



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *
from djangovideos import views

urlpatterns = patterns('',
    url(r'^add_video/(?P<app_label>[\w\-]+)/(?P<module_name>[\w\-]+)/(?P<pk>\d+)/$',
        views.add_video, name='add_video'),
    url(r'^delete_video/(?P<video_pk>\d+)/$',
        views.delete_video, name='delete_video'),

)
